#include "extensionswindow.h"
#include <vector>

enum
{
	BTN_ADD = wxID_HIGHEST,
	BTN_CHANGE,
	BTN_DELETE,
	BTN_CLOSE,
	O_FILE_LIST,
	
};

BEGIN_EVENT_TABLE(ExtensionsWindow, wxFrame)
	EVT_BUTTON(BTN_ADD, ExtensionsWindow::on_btnAdd)
	EVT_BUTTON(BTN_CHANGE, ExtensionsWindow::on_btnChange)
	EVT_BUTTON(BTN_DELETE, ExtensionsWindow::on_btnDelete)
	EVT_BUTTON(BTN_CLOSE, ExtensionsWindow::on_btnClose)
	EVT_LISTBOX(O_FILE_LIST, ExtensionsWindow::on_oFileList)
	EVT_CLOSE(ExtensionsWindow::on_this_close)
END_EVENT_TABLE()

ExtensionsWindow::ExtensionsWindow(wxWindow *oParent, const wxString &sTitle)
: wxFrame(oParent, -1, sTitle, wxDefaultPosition, wxDefaultSize)
{
	oPanel = new wxPanel(this, -1);
	oConfig = (wxConfig *)wxConfig::Get();
	
	// Sizer creation.
	oWindowSizer = new wxBoxSizer(wxVERTICAL);
	oMainSizer = new wxBoxSizer(wxVERTICAL);
	oSettingsSizer = new wxBoxSizer(wxHORIZONTAL);
	oInputSizer = new wxFlexGridSizer(2, 3);
	oButtonSizer = new wxBoxSizer(wxHORIZONTAL);
	oExtensionSizer = new wxBoxSizer(wxVERTICAL);
	
	oInputSizer->AddGrowableCol(1);
	
	// Control creation.
	oFileList = new wxListBox(oPanel, O_FILE_LIST);
	
	lblType = new wxStaticText(oPanel, -1, _("Type: File"));
	lblTypeValue = new wxStaticText(oPanel, -1, _(""));
	
	lblEnding = new wxStaticText(oPanel, -1, _("Ending:"));
	lblRun = new wxStaticText(oPanel, -1, _("Run:"));
	lblTerminal = new wxStaticText(oPanel, -1, _(""));
	
	txtEnding = new wxTextCtrl(oPanel, -1);
	txtRun = new wxTextCtrl(oPanel, -1);
	btnTerminal = new wxCheckBox(oPanel, -1, _("Show output"));
	
	btnAdd = new wxButton(oPanel, BTN_ADD, _("Add"));
	btnChange = new wxButton(oPanel, BTN_CHANGE, _("Change"));
	btnDelete = new wxButton(oPanel, BTN_DELETE, _("Delete"));
	
	btnClose = new wxButton(oPanel, BTN_CLOSE, _("Close"));
	
	// Sizer manipulation.
	oButtonSizer->Add(btnAdd, 1, wxALL, 4);
	oButtonSizer->Add(btnChange, 1, wxALL, 4);
	oButtonSizer->Add(btnDelete, 1, wxALL, 4);
	
	oInputSizer->Add(lblType, 0, wxALL, 4);
	oInputSizer->Add(lblTypeValue, 1, wxGROW | wxTOP, 4);
	oInputSizer->Add(lblEnding, 0, wxALL, 4);
	oInputSizer->Add(txtEnding, 1, wxGROW | wxRIGHT, 4);
	oInputSizer->Add(lblRun, 0, wxALL, 4);
	oInputSizer->Add(txtRun, 1, wxGROW | wxRIGHT, 4);
	oInputSizer->Add(lblTerminal, 0, wxALL, 4);
	oInputSizer->Add(btnTerminal, 1, wxGROW | wxRIGHT, 4);
	
	oExtensionSizer->Add(oInputSizer, 0, wxGROW);
	oExtensionSizer->Add(oButtonSizer, 0, wxGROW);
	
	oSettingsSizer->Add(oFileList, 0, wxGROW);
	oSettingsSizer->Add(oExtensionSizer, 1, wxGROW | wxLEFT, 4);
	
	oMainSizer->Add(oSettingsSizer, 1, wxGROW);
	oMainSizer->Add(btnClose, 0, wxALIGN_RIGHT | wxALL, 4);
	
	oWindowSizer->Add(oMainSizer, wxALL | wxGROW, 4);
	
	// Formalities.
	oPanel->SetSizer(oWindowSizer);
	oPanel->SetAutoLayout(true);
	oWindowSizer->Fit(this);
	
	// Finally...
	readSettings();
}

void ExtensionsWindow::updateEntity()
{
	wxString sEnding = txtEnding->GetValue();
	if (sEnding.IsEmpty())
	{
		return;
	}
	
	oConfig->Write(_("/Files/") + sEnding + _("/Run"), txtRun->GetValue());
	oConfig->Write(_("/Files/") + sEnding + _("/RunInTerminal"), btnTerminal->GetValue() ? _("1") : _("0"));
	
	readSettings();
	oFileList->SetStringSelection(sEnding);
	
	txtEnding->SetValue(_(""));
	txtRun->SetValue(_(""));
	btnTerminal->SetValue(false);
	
	txtEnding->SetFocus();
}

void ExtensionsWindow::on_btnAdd(wxCommandEvent &oEvent)
{
	updateEntity();
}

void ExtensionsWindow::on_btnChange(wxCommandEvent &oEvent)
{
	updateEntity();
}

void ExtensionsWindow::on_btnDelete(wxCommandEvent &oEvent)
{
	wxString sEnding = txtEnding->GetValue();
	if (sEnding.IsEmpty())
	{
		return;
	}
	
	oConfig->DeleteGroup(_("/Files/") + sEnding);
	
	readSettings();
	txtEnding->SetValue(_(""));
	txtRun->SetValue(_(""));
	txtEnding->SetFocus();
}

void ExtensionsWindow::readSettings()
{
	wxString sEntry;
	long iDummy = 0;
	
	oConfig->SetPath(_("/Files"));
	
	oFileList->Clear();
	
	bool bCont = oConfig->GetFirstGroup(sEntry, iDummy);
	while (bCont) {
		oFileList->Append(sEntry);
		
		bCont = oConfig->GetNextGroup(sEntry, iDummy);
	}
}

void ExtensionsWindow::loadSelection(const wxString &sExtension)
{
	txtEnding->SetValue(sExtension);
	txtRun->SetValue(oConfig->Read(_("/Files/") + sExtension + _("/Run")));
	btnTerminal->SetValue(oConfig->Read(_("/Files/") + sExtension + _("/RunInTerminal")) == _("1"));
	txtEnding->SetFocus();
}

void ExtensionsWindow::on_oFileList(wxCommandEvent &oEvent)
{
	loadSelection(oFileList->GetStringSelection());
}

void ExtensionsWindow::on_btnClose(wxCommandEvent &oEvent)
{
	Close();
}

void ExtensionsWindow::on_this_close(wxCloseEvent &oEvent)
{
	readSettings();
	Show(false);
}


