#ifndef FILETREE_H
#define FILETREE_H
#include <wx/wx.h>
#include <wx/treectrl.h>
#include <wx/image.h>
#include <wx/dir.h>
#include <wx/file.h>
#include <wx/config.h>
#include <vector>

#include "mainwindow.h"
#include "dropsource.h"

class MainWindow;
class UThread;

class UFiletree : public wxTreeCtrl
{
public:
	UFiletree(wxWindow *oParent, wxWindowID iID, wxTextCtrl *oLocation, MainWindow *oParentWindow);
	
	void dropFiles(std::vector<wxString> sFiles, wxCoord oX, wxCoord oY);
	wxString getFullpath(wxTreeItemId iItem);
	void selectPrevItem();
	void selectNextItem();
	wxTreeItemId getPrevID(const wxTreeItemId &iItem); // wxWidget's GetPrevVisible() still doesn't work correctly. 2006-06-10
	void ReloadDir(wxTreeItemId iItemID,bool bCheckOnly);
	wxTreeItemId addItem(wxTreeItemId iDirID, const wxString &sItem);
	void SetRoot(const wxString &sDir, bool bSkipLocationText);
	void Activate(wxTreeItemId iItemID);
	void Activate(wxString sItem);
	bool addFile();
	bool addFolder();
	bool Rename(wxTreeItemId iItemID, const wxString &sNewName);
	bool deleteItem();
	void GoUp();
	void rightPress();
	void leftPress();
	void selectTreeItem(wxTreeItemId iItem);
	void pressHome();
	void pressEnd();
	void openWith();
	void edit();
	bool somethingSelected();
	bool itemExists(const wxTreeItemId &iItem);
	bool itemExists(const wxString &sItem);
	void fixSelection();
	wxTreeItemId getSingleTreeSelection();
	std::vector<wxTreeItemId> getTreeSelections();
	void logMessage(const wxString &sMessage);
	void addValid(wxTreeItemId iItem);
	void delValid(wxTreeItemId iItem);
	bool isValid(wxTreeItemId iItem);
	std::vector<wxTreeItemId> iValidItems;
	void delAllValids();
	void createThread();
	std::vector<wxString> split(const wxString &sString, char cSplit);
	
	bool move(const wxString &sOriginal, const wxString &sDestination);
	bool isExecutable(const wxString &sFilename);
	bool isSymlink(const wxString &sFilename);
	
	bool bSeeHidden;
	wxString sSelectedItem;
	
	wxConfig *oConfig;
	MainWindow *oParentFrame;
	
private:
	UDropSource *oDropSource;
	wxTextCtrl *txtLocation;
	wxTreeItemId iMovingID;
	wxString sTempItemLabel;
	UThread *oThread;
	
	void OnDragBegin(wxTreeEvent &oEvent);
	void OnDragEnd(wxTreeEvent &oEvent);
	void OnItemExpanding(wxTreeEvent &oEvent);
	void OnItemCollapsing(wxTreeEvent &oEvent);
	void OnItemCollapsed(wxTreeEvent &oEvent);
	void OnItemActivated(wxTreeEvent &oEvent);
	void OnRightClick(wxTreeEvent &oEvent);
	void OnItemBeginRename(wxTreeEvent &oEvent);
	void OnItemEndRename(wxTreeEvent &oEvent);
	void OnSelChanged(wxFocusEvent &oEvent);

	void OnLeaveWindow(wxMouseEvent &oEvent);

	void run(const wxString &sCommand);
	void run(const wxString &sCommand, bool bTerminal);
	
	DECLARE_EVENT_TABLE()
};

#endif

