#include <wx/image.h>

#include "mainwindow.h"

#include "images/file.xpm"
#include "images/app.xpm"
#include "images/app_symlink.xpm"
#include "images/file_symlink.xpm"
#include "images/file_pd.xpm"
#include "images/folder.xpm"
#include "images/folder_pd.xpm"
#include "images/folder_symlink.xpm"
#include "images/up.xpm"
#include "images/home.xpm"
#include "images/fit.xpm"

enum
{
	O_TREE = wxID_HIGHEST,
	O_TIMER,
	TXT_LOCATION,
	TXT_RUN,
	MNU_NEWFOLDER,
	MNU_NEWFILE,
	MNU_OPENANDVIEW,
	MNU_OPENINNEW,
	MNU_OPENWITH,
	MNU_DELETE,
	MNU_EXIT,
	MNU_RENAME,
	MNU_HIDDEN,
	MNU_SHOW_MENUBAR,
	MNU_SHOW_LOCATION,
	MNU_SHOW_TOOLBAR,
	MNU_AUTO_FIT_TREE,
	MNU_UP,
	MNU_HOME,
	MNU_TREEFIT,
	MNU_SETTINGS,
	MNU_EXTENSIONS,
	BTN_PROCESS_CLOSE,
	BTN_UP,
	BTN_HOME,
	BTN_FIT,
};

BEGIN_EVENT_TABLE(MainWindow, wxFrame)
	// EVT_TIMER(O_TIMER, MainWindow::OnTimer) // To kill?
	EVT_TEXT(TXT_LOCATION, MainWindow::OnLocationChange)
	EVT_TEXT_ENTER(TXT_LOCATION, MainWindow::OnLocationEnter)
	EVT_CHAR(MainWindow::OnLocationTab)
	EVT_MENU(MNU_UP, MainWindow::OnUp)
	EVT_MENU(MNU_HOME, MainWindow::OnHome)
	EVT_MENU(MNU_NEWFOLDER, MainWindow::OnMenuNewFolder)
	EVT_MENU(MNU_NEWFILE, MainWindow::OnMenuNewFile)
	EVT_MENU(MNU_OPENANDVIEW, MainWindow::OnMenuOpenAndView)
	EVT_MENU(MNU_OPENINNEW, MainWindow::OnMenuOpenInNew)
	EVT_MENU(MNU_OPENWITH, MainWindow::OnMenuOpenWith)
	EVT_MENU(MNU_EXIT, MainWindow::OnMenuExit)
	EVT_MENU(MNU_RENAME, MainWindow::OnMenuRename)
	EVT_MENU(MNU_DELETE, MainWindow::OnMenuDelete)
	EVT_MENU(MNU_SHOW_MENUBAR, MainWindow::OnMenuShowMenubar)
	EVT_MENU(MNU_SHOW_LOCATION, MainWindow::OnMenuShowLocation)
	EVT_MENU(MNU_SHOW_TOOLBAR, MainWindow::OnMenuShowToolbar)
	EVT_MENU(MNU_AUTO_FIT_TREE, MainWindow::OnMenuAutoFitTree)
	EVT_MENU(MNU_HIDDEN, MainWindow::OnMenuHidden)
	EVT_MENU(MNU_SETTINGS, MainWindow::OnMenuSettings)
	EVT_MENU(MNU_EXTENSIONS, MainWindow::OnMenuExtensions)
	EVT_TREE_KEY_DOWN(O_TREE, MainWindow::OnTreeKeyDown)
	EVT_TREE_ITEM_RIGHT_CLICK(O_TREE, MainWindow::OnRightClick)
	EVT_BUTTON(BTN_PROCESS_CLOSE, MainWindow::OnButtonProcessClose)
	EVT_BUTTON(BTN_UP, MainWindow::OnUp)
	EVT_BUTTON(BTN_HOME, MainWindow::OnHome)
	EVT_BUTTON(BTN_FIT, MainWindow::OnFit)
	EVT_MOVE(MainWindow::OnMove)
END_EVENT_TABLE()

MainWindow::MainWindow(const wxString &sTitle)
: wxFrame((wxFrame *)NULL, -1, sTitle, wxPoint(0, 0), wxSize(300, 400), wxDEFAULT_FRAME_STYLE, _("jaffm"))
{
	bLocationVisible = false;
	bToolbarVisible = false;
	bAutoFitTree = false;
	
	wxString sBaseDir(wxGetCwd());
	long iInterval = 400;
	
	oConfig = (wxConfig *)wxConfig::Get();
	
	oPanel = new wxPanel(this, -1);
	
	if (wxTheApp->argc > 1)
	{
		if (wxDirExists(wxTheApp->argv[1]) && wxString(wxTheApp->argv[1]) != _("."))
		{
			sBaseDir = wxTheApp->argv[1];
		}
		if (wxTheApp->argc > 2)
		{
			wxString(wxTheApp->argv[2]).ToLong(&iInterval);
		}
	}
	
	oMainSizer = new wxBoxSizer(wxVERTICAL);
	
	// Toolbar.
	oToolbarSizer = new wxBoxSizer(wxHORIZONTAL);
	wxBitmap imgUp  = (const char **)up_xpm;
	wxBitmap imgHome = (const char **)home_xpm;
	wxBitmap imgFit = (const char **)fit_xpm;
	btnUp = new wxBitmapButton(oPanel, BTN_UP, imgUp);
	btnHome = new wxBitmapButton(oPanel, BTN_HOME, imgHome);
	btnFit = new wxBitmapButton(oPanel, BTN_FIT, imgFit);
	btnUp->Show(false);
	btnHome->Show(false);
	btnFit->Show(false);
	
	// Resizing toolbar buttons to get rid of most of their borders.
	wxSize oTempSize;
	oTempSize = wxSize(btnUp->GetSize().GetWidth() - 6, btnUp->GetSize().GetHeight() - 6);
	btnUp->SetSize(oTempSize);
	btnHome->SetSize(oTempSize);
	btnFit->SetSize(oTempSize);
	
	// Assigning toolbar buttons to toolbar.
	oToolbarSizer->Add(btnUp, 0, wxALL, 2);
	oToolbarSizer->Add(btnHome, 0, wxALL, 2);
	oToolbarSizer->Add(btnFit, 0, wxALL, 2);
	
	txtLocation = new wxTextCtrl(oPanel, TXT_LOCATION, sBaseDir, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	txtLocation->Show(false);
	
	oProcessBox = new UProcessBox(oPanel);
	oProcessBox->Show(false);
	
	btnProcessClose = new wxButton(oPanel, BTN_PROCESS_CLOSE, _("Close"));
	btnProcessClose->Show(false);
	
	wxDir oDir(wxGetCwd());
	
	wxString sFilename;
	
	oTree = new UFiletree(oPanel, O_TREE, txtLocation, this);
	
	oImageList = new wxImageList(16, 16);
	
	wxBitmap imgFile;
	wxBitmap imgFolder;
	wxBitmap imgApp;
	wxBitmap imgFolder_PD;
	wxBitmap imgFile_Symlink;
	wxBitmap imgApp_Symlink;
	wxBitmap imgFolder_Symlink;
	wxBitmap imgFile_PD;
	
	imgFile = (const char **)file_xpm;
	imgFolder = (const char **)folder_xpm;
	imgApp = (const char **)app_xpm;
	imgFolder_PD = (const char **)folder_pd_xpm;
	imgFile_Symlink = (const char **)file_symlink_xpm;
	imgApp_Symlink = (const char **)app_symlink_xpm;
	imgFolder_Symlink = (const char **)folder_symlink_xpm;
	imgFile_PD = (const char **)file_pd_xpm;
	
	oImageList->Add(imgFile);
	oImageList->Add(imgFolder);
	oImageList->Add(imgApp);
	oImageList->Add(imgFolder_PD);
	oImageList->Add(imgFile_Symlink);
	oImageList->Add(imgApp_Symlink);
	oImageList->Add(imgFolder_Symlink);
	oImageList->Add(imgFile_PD);
	
	oTree->SetImageList(oImageList);
	
	wxIcon oIcon;
	oIcon.CopyFromBitmap(imgFolder);
	SetIcon(oIcon);
	
	oActionsMenu = new wxMenu();
	oActionsMenu->Append(MNU_NEWFOLDER, _("New Folder (&D)"));
	oActionsMenu->Append(MNU_NEWFILE, _("New File (&F)"));
	oActionsMenu->AppendSeparator();
	oActionsMenu->Append(MNU_OPENANDVIEW, _("Open And View Output"));
	oActionsMenu->Append(MNU_OPENINNEW, _("Open In New Window (&W)"));
	oActionsMenu->Append(MNU_OPENWITH, _("Open With... (&O)"));
	oActionsMenu->AppendSeparator();
	oActionsMenu->Append(MNU_RENAME, _("Rename (&R)"));
	oActionsMenu->Append(MNU_DELETE, _("Delete (D&el)"));
	oActionsMenu->AppendSeparator();
	oActionsMenu->Append(MNU_EXIT, _("E&xit (Esc)"));
	
	oMainSizer->Add(oTree, 1, wxGROW | wxALL, 2);
	
	oNavMenu = new wxMenu();
	
	mnuShowHidden = new wxMenuItem(oNavMenu, MNU_HIDDEN, _("Show Hidden (&H)"), wxEmptyString, true);
	mnuShowMenubar = new wxMenuItem(oNavMenu, MNU_SHOW_MENUBAR, _("Show Menubar (&M)"), wxEmptyString, true);
	mnuShowLocation = new wxMenuItem(oNavMenu, MNU_SHOW_LOCATION, _("Show Location (&L)"), wxEmptyString, true);
	mnuShowToolbar = new wxMenuItem(oNavMenu, MNU_SHOW_TOOLBAR, _("Show Toolbar (&T)"), wxEmptyString, true);
	mnuAutoFitTree = new wxMenuItem(oNavMenu, MNU_AUTO_FIT_TREE, _("Auto fit tree (&A)"), wxEmptyString, true);
	
	oNavMenu->Append(MNU_UP, _("Go Up (&Backspace)"));
	oNavMenu->Append(MNU_HOME, _("Go Home (&U)"));
	oNavMenu->AppendSeparator();
	oNavMenu->Append(mnuShowHidden);
	oNavMenu->Append(mnuShowMenubar);
	oNavMenu->Append(mnuShowLocation);
	oNavMenu->Append(mnuShowToolbar);
	oNavMenu->Append(mnuAutoFitTree);
	oNavMenu->AppendSeparator();
	oNavMenu->Append(MNU_SETTINGS, _("&Settings"));
	oNavMenu->Append(MNU_EXTENSIONS, _("File &Extensions"));
	
	oMenuBar = new wxMenuBar();
	oMenuBar->Append(oActionsMenu, _("&Actions"));
	oMenuBar->Append(oNavMenu, _("&Navigation"));
	
	oPanel->SetSizer(oMainSizer);
	oPanel->SetAutoLayout(true);
	
	oTree->SetRoot(sBaseDir, true);
	
	if (oConfig->Read(_("/ShowHidden")) == _("1")) {
		toggleHidden();
	}
	if (oConfig->Read(_("/ShowMenubar")) == _("1"))
	{
		toggleMenubar();
	}
	if (oConfig->Read(_("/ShowLocation")) == _("1"))
	{
		toggleLocationBar();
	}
	if (oConfig->Read(_("/ShowToolbar")) == _("1"))
	{
		toggleToolbar();
	}
	if (oConfig->Read(_("/AutoFitTree")) == _("1"))
	{
		toggleAutoFit();
	}
	
	CreateStatusBar();
	
	oSettingsWindow = new SettingsWindow(this, _("Settings"));
	oExtensionsWindow = new ExtensionsWindow(this, _("File Extensions"));
	
	treeFit(true);
	
	// oTimer = new wxTimer(this, O_TIMER);
	// oTimer->Start(iInterval);
}

void MainWindow::OnUp(wxCommandEvent &oEvent)
{
	oTree->GoUp();
}

void MainWindow::OnMenuRename(wxCommandEvent &oEvent)
{
	oTree->edit();
}

void MainWindow::OnMenuDelete(wxCommandEvent &oEvent)
{
	oTree->deleteItem();
}

void MainWindow::OnMenuNewFolder(wxCommandEvent &oEvent)
{
	oTree->addFolder();
}

void MainWindow::OnMenuNewFile(wxCommandEvent &oEvent)
{
	oTree->addFile();
}

void MainWindow::OnMenuShowMenubar(wxCommandEvent &oEvent)
{
	toggleMenubar();
}

void MainWindow::OnMenuShowLocation(wxCommandEvent &oEvent)
{
	toggleLocationBar();
}

void MainWindow::OnMenuShowToolbar(wxCommandEvent &oEvent)
{
	toggleToolbar();
}

void MainWindow::OnMenuAutoFitTree(wxCommandEvent &oEvent)
{
	toggleAutoFit();
}

void MainWindow::toggleMenubar()
{
	if (GetMenuBar() != NULL)
	{
		SetMenuBar(NULL);

		oNavMenu->Check(mnuShowMenubar->GetId(), false);
		
		oConfig->Write(_("/ShowMenubar"), _("0"));
	}
	else
	{
		SetMenuBar(oMenuBar);

		oNavMenu->Check(mnuShowMenubar->GetId(), true);

		oConfig->Write(_("/ShowMenubar"), _("1"));
	}
}

void MainWindow::toggleToolbar()
{
	if (bToolbarVisible)
	{
		btnUp->Show(false);
		btnHome->Show(false);
		btnFit->Show(false);
		
		oNavMenu->Check(mnuShowToolbar->GetId(), false);
		
		oMainSizer->Remove(oToolbarSizer);
		
		oConfig->Write(_("/ShowToolbar"), _("0"));
		bToolbarVisible = false;
	}
	else
	{
		oToolbarSizer = new wxBoxSizer(wxHORIZONTAL);
		oToolbarSizer->Add(btnUp, 0, wxALL, 2);
		oToolbarSizer->Add(btnHome, 0, wxALL, 2);
		oToolbarSizer->Add(btnFit, 0, wxALL, 2);
		
		btnUp->Show(true);
		btnHome->Show(true);
		btnFit->Show(true);
		
		oNavMenu->Check(mnuShowToolbar->GetId(), true);
		
		oMainSizer->Prepend(oToolbarSizer);
		
		oConfig->Write(_("/ShowToolbar"), _("1"));
		bToolbarVisible = true;
	}
	oMainSizer->Layout();
}

void MainWindow::toggleLocationBar()
{
	if (bLocationVisible)
	{
		txtLocation->Show(false);
		
		oNavMenu->Check(mnuShowLocation->GetId(), false);
		
		oMainSizer->Remove(oLocationSizer);
		
		oConfig->Write(_("/ShowLocation"), _("0"));
		bLocationVisible = false;
		
		oTree->SetFocus();
	}
	else
	{
		oLocationSizer = new wxBoxSizer(wxHORIZONTAL);
		oLocationSizer->Add(txtLocation, 1, wxGROW);
		
		txtLocation->Show(true);
		
		oNavMenu->Check(mnuShowLocation->GetId(), true);
		
		oMainSizer->Prepend(oLocationSizer, 0, wxGROW | wxALL, 2);
		
		txtLocation->SetFocus();
		txtLocation->SetInsertionPointEnd();
		
		oConfig->Write(_("/ShowLocation"), _("1"));
		bLocationVisible = true;
	}
	oMainSizer->Layout();
}

void MainWindow::OnMenuHidden(wxCommandEvent &oEvent)
{
	toggleHidden();
}

void MainWindow::toggleAutoFit()
{
	if (bAutoFitTree)
	{
		mnuAutoFitTree->Check(false);
		oConfig->Write(_("/AutoFitTree"), _("0"));
		bAutoFitTree = false;
	}
	else
	{
		mnuAutoFitTree->Check(true);
		oConfig->Write(_("/AutoFitTree"), _("1"));
		bAutoFitTree = true;
	}
}

void MainWindow::toggleHidden()
{
	if (oTree->bSeeHidden)
	{
		oNavMenu->Check(mnuShowHidden->GetId(), false);
		
		oConfig->Write(_("/ShowHidden"), _("0"));
		oTree->bSeeHidden = false;
	}
	else
	{
		oNavMenu->Check(mnuShowHidden->GetId(), true);
		
		oConfig->Write(_("/ShowHidden"), _("1"));
		oTree->bSeeHidden = true;
	}
	
	oTree->SetRoot(oTree->GetItemText(oTree->GetRootItem()), true);
}

void MainWindow::OnMenuSettings(wxCommandEvent &oEvent)
{
	oSettingsWindow->Show(true);
}

void MainWindow::OnMenuExtensions(wxCommandEvent &oEvent)
{
	oExtensionsWindow->Show(true);
}

void MainWindow::OnLocationEnter(wxCommandEvent &oEvent)
{
	wxTreeItemId iCurrent = oTree->GetRootItem();
	while (iCurrent = oTree->GetNextVisible(iCurrent))
	{
		oTree->SetItemBold(iCurrent, false);
	}
	
	if (wxFile::Exists(txtLocation->GetValue()))
	{
		oTree->Activate(txtLocation->GetValue());
	}
	oTree->SetFocus();
	toggleLocationBar();
}

void MainWindow::OnLocationChange(wxCommandEvent &oEvent)
{
	wxString sItemText;
	wxString sValue = txtLocation->GetValue();
	wxString sBasename = sValue.Mid(sValue.Find('/', true) + 1);
	
	wxTreeItemId iCurrent = oTree->GetRootItem();
	while (iCurrent = oTree->GetNextVisible(iCurrent))
	{
		sItemText = oTree->GetItemText(iCurrent);
		if (sBasename.Length() != 0)
		{
			if (sItemText.Mid(0, sBasename.Length()) == sBasename)
			{
				oTree->SetItemBold(iCurrent, true);
			}
			else
			{
				oTree->SetItemBold(iCurrent, false);
			}
		}
	}
	
	if (wxDirExists(txtLocation->GetValue()))
	{
		oTree->SetRoot(sValue, true);
	}
}

void MainWindow::OnLocationTab(wxKeyEvent &oEvent)
{
	if (oEvent.GetKeyCode() != WXK_TAB)
	{
		return;
	}
	
	wxString sItemText;
	wxString sValue = txtLocation->GetValue();
	wxString sBasename = sValue.Mid(sValue.Find('/', true) + 1);
	
	std::vector<wxString> sPossibleItems;
	
	wxTreeItemId iCurrent = oTree->GetRootItem();
	while (iCurrent = oTree->GetNextVisible(iCurrent))
	{
		sItemText = oTree->GetItemText(iCurrent);
		if (sBasename.Length() != 0)
		{
			if (sItemText.Mid(0, sBasename.Length()) == sBasename)
			{
				sPossibleItems.push_back(sItemText);
			}
		}
	}
	
	int iLength = sBasename.Length();
	
	if (sPossibleItems.size() > 0)
	{
		unsigned int iMaxLength = 0;
		for (unsigned int i = 0; i < sPossibleItems.size(); i++)
		{
			if (sPossibleItems[i].Length() > iMaxLength)
			{
				iMaxLength = sPossibleItems[i].Length();
			}
		}
		
		wxString sRestOfName;
		wxString sFirstChar;
		wxString sNextChar;
		bool bDone = false;
		for (unsigned int i = iLength; i < iMaxLength; i++)
		{
			sFirstChar = sPossibleItems[0].Mid(i, 1);
			bDone = false;
			for (unsigned int ii = 0; ii < sPossibleItems.size(); ii++)
			{
				if (sPossibleItems[ii].Mid(i, 1) != sFirstChar)
				{
					bDone = true;
				}
			}
			if (!bDone)
			{
				sNextChar = sFirstChar;
			}
			else
			{
				break;
			}
			sRestOfName += sNextChar;
		}
		txtLocation->SetValue(txtLocation->GetValue() + sRestOfName);
	}
	
	wxString sLocation = txtLocation->GetValue();
	
	if (txtLocation->GetValue().Right(1) != _("/") && wxDirExists(sLocation))
	{
		txtLocation->SetValue(sLocation + _("/"));
	}
	
	txtLocation->SetInsertionPointEnd();
}

void MainWindow::OnMenuExit(wxCommandEvent &oEvent)
{
	Close();
}

void MainWindow::OnMenuOpenInNew(wxCommandEvent &oEvent)
{
	openInNew();
}

void MainWindow::openInNew()
{
	wxString sPath;
	wxString sRunString;
	wxString sAppCommand;
	
	sAppCommand = wxTheApp->argv[0];
	sPath = oTree->getFullpath(oTree->getSingleTreeSelection());
	sRunString = sAppCommand + _(" ") + sPath + _(" &");
	system((const char *)sRunString.mb_str());
}

void MainWindow::treeFit(bool bForce)
{
	if (FindFocus() != oTree && !bForce)
	{
		return;
	}
	
	// Finding out how big the screen is.
	int iScreenWidth, iScreenHeight;
	wxDisplaySize(&iScreenWidth, &iScreenHeight);
	
	// Finding out how big the tree is.
	int iTreeWidth, iTreeHeight;
	oTree->GetSize(&iTreeWidth, &iTreeHeight);
	
	// Finding out how big our window is.
	// We'll need this later in order to determine the difference between the window and the tree.
	int iWinWidth, iWinHeight;
	GetSize(&iWinWidth, &iWinHeight);
	
	// Here we just check to see how large any object is in the tree.
	// It can vary between icons, text size and so on.
	// We assume that every line is exactly as high (hihihi) as the root item.
	wxRect oTreeRect;
	oTree->GetBoundingRect(oTree->GetRootItem(), oTreeRect);
	int iItemHeight = oTreeRect.GetHeight();
	
	// This is how big the tree must be in order to lose the scrollbar.
	int iNewTreeHeight = (iItemHeight * oTree->GetCount()) + 15;
	
	int iDiffHeight = iNewTreeHeight - iTreeHeight;
	
	// This defines how much margin space we want.
	// IMPORTANT: We really shouldn't be reading this from the configuration every time the tree resizes itself.
	// The wxConfig class is neither designed for low memory consumption and certainly not speed,
	// so iScreenMargin should be a part of MainWindow and be updated whenever the configuration changes.
	// Long story short, this mechanism should go to where the screen margin is updated.
	wxString sScreenMargin(oConfig->Read(_("/ScreenMargin")));
	long iScreenMargin;
	if (!sScreenMargin.ToLong(&iScreenMargin))
	{
		iScreenMargin = 100;
	}
	
	int iMaxTreeHeight = iScreenHeight - GetRect().GetTop() - iScreenMargin;
	
	// Here we take the process box into account.
	if (oProcessBox->IsShown())
	{
		int iProcessBoxWidth, iProcessBoxHeight;
		oProcessBox->GetSize(&iProcessBoxWidth, &iProcessBoxHeight);
		iMaxTreeHeight -= iProcessBoxHeight;
	}

	// Here we take the status bar into account.
	int iStatusBarWidth, iStatusBarHeight;
	GetStatusBar()->GetSize(&iStatusBarWidth, &iStatusBarHeight);
	iMaxTreeHeight -= iStatusBarHeight;
	
	// Here we make sure that the window doesn't get bigger than the screen.
	// We'll rather want a scrollbar than that.
	if (iNewTreeHeight > iMaxTreeHeight)
	{
		iNewTreeHeight = iMaxTreeHeight;
		iDiffHeight = iNewTreeHeight - iTreeHeight;
	}
	
	// And finally, we set the size.
	if (iDiffHeight != 0)
	{
		oTree->SetSize(iTreeWidth, iNewTreeHeight);
		SetSize(iWinWidth, iWinHeight + iDiffHeight);
	}
}

// To kill?
/*
void MainWindow::OnTimer(wxTimerEvent &oEvent)
{
	oTimer->Stop();
	
	if (bAutoFitTree)
	{
		treeFit();
	}
	
	oTimer->Start();
}
*/

void MainWindow::OnTreeKeyDown(wxTreeEvent &oEvent)
{
	// We send the keycode to getKeySignal because we're preparing for user-defined keys.
	wxString sKeySignal = getKeySignal(oEvent.GetKeyCode());
	
	if (sKeySignal == _("ToggleMenubar"))
	{
		toggleMenubar();
	}
	else if (sKeySignal == _("EditLabel"))
	{
		oTree->EditLabel(oTree->getSingleTreeSelection());
	}
	else if (sKeySignal == _("DeleteItem"))
	{
		oTree->deleteItem();
	}
	else if (sKeySignal == _("AddFile"))
	{
		oTree->addFile();
	}
	else if (sKeySignal == _("ToggleLocationBar"))
	{
		toggleLocationBar();
	}
	else if (sKeySignal == _("ToggleHidden"))
	{
		toggleHidden();
	}
	else if (sKeySignal == _("AddFolder"))
	{
		oTree->addFolder();
	}
	else if (sKeySignal == _("OpenWith"))
	{
		oTree->openWith();
	}
	else if (sKeySignal == _("Close"))
	{
		Close();
	}
	else if (sKeySignal == _("GoUp"))
	{
		oTree->GoUp();
	}
	else if (sKeySignal == _("PressEnd"))
	{
		oTree->pressEnd();
	}
	else if (sKeySignal == _("PressHome"))
	{
		oTree->pressHome();
	}
	else if (sKeySignal == _("LeftPress"))
	{
		oTree->leftPress();
	}
	else if (sKeySignal == _("SelectPrevItem"))
	{
		oTree->selectPrevItem();
	}
	else if (sKeySignal == _("RightPress"))
	{
		oTree->rightPress();
	}
	else if (sKeySignal == _("SelectNextItem"))
	{
		oTree->selectNextItem();
	}
	else if (sKeySignal == _("Activate"))
	{
		oTree->Activate(oTree->getSingleTreeSelection());
	}
	else if (sKeySignal == _("OpenInNew"))
	{
		openInNew();
	}
	else if (sKeySignal == _("TreeFit"))
	{
		treeFit();
	}
	else if (sKeySignal == _("GoHome"))
	{
		goHome();
	}
	else if (sKeySignal == _("AutoFitTree"))
	{
		toggleAutoFit();
	}
	else if (sKeySignal == _("ToggleToolbar"))
	{
		toggleToolbar();
	}
}

void MainWindow::OnRightClick(wxTreeEvent &oEvent)
{
	oTree->selectTreeItem(oEvent.GetItem());
	PopupMenu(oActionsMenu, oEvent.GetPoint());
}

void MainWindow::OnMenuOpenWith(wxCommandEvent &oEvent)
{
	oTree->openWith();
}

void MainWindow::OnMenuOpenAndView(wxCommandEvent &oEvent)
{
	wxString sFullpath;
	sFullpath = oTree->getFullpath(oTree->getSingleTreeSelection());
	if (!oTree->isExecutable(sFullpath))
	{
		return;
	}
	
	oProcessBox->setCommand(sFullpath);
	
	if (!oProcessBox->IsShown())
	{
		oProcessBox->Show(true);
		btnProcessClose->Show(true);
		oMainSizer->Add(oProcessBox, 0, wxGROW);
		oMainSizer->Add(btnProcessClose, 0, wxALIGN_CENTER, wxALIGN_CENTER);
		oMainSizer->SetItemMinSize(oProcessBox, 10, 120);
		oMainSizer->Layout();
		btnProcessClose->SetFocus();
	}
	
	oProcessBox->run();
}

void MainWindow::OnButtonProcessClose(wxCommandEvent &oEvent)
{
	btnProcessClose->Show(false);
	oProcessBox->Show(false);
	oMainSizer->Remove(btnProcessClose);
	oMainSizer->Remove(oProcessBox);
	oMainSizer->Layout();
	treeFit();
	oTree->SetFocus();
}

void MainWindow::OnHome(wxCommandEvent &oEvent)
{
	goHome();
}

void MainWindow::goHome()
{
	oTree->SetRoot(wxGetUserHome(), false);
	oTree->SetFocus();
}

void MainWindow::OnFit(wxCommandEvent &oEvent)
{
	oTree->SetFocus();
	treeFit();
}


wxString MainWindow::getKeySignal(int iCode)
{
	wxString sReturn;
	
	// Just for testing.
	// printf("Key: %c, %d\n", iCode, iCode);
	
	if (iCode == 'm')
	{
		sReturn = _("ToggleMenubar");
	}
	else if (iCode == 'r')
	{
		sReturn = _("EditLabel");
	}
	else if (iCode == 127) // Delete
	{
		sReturn = _("DeleteItem");
	}
	else if (iCode == 'f')
	{
		sReturn = _("AddFile");
	}
	else if (iCode == 'l')
	{
		sReturn = _("ToggleLocationBar");
	}
	else if (iCode == 'h')
	{
		sReturn = _("ToggleHidden");
	}
	else if (iCode == 'd')
	{
		sReturn = _("AddFolder");
	}
	else if (iCode == 111)
	{
		sReturn = _("OpenWith");
	}
	else if (iCode == 27) // Escape
	{
		sReturn = _("Close");
	}
	else if (iCode == 8) // Backspace
	{
		sReturn = _("GoUp");
	}
	else if (iCode == 314) // End
	{
		sReturn = _("PressEnd");
	}
	else if (iCode == 315) // Home
	{
		sReturn = _("PressHome");
	}
	else if (iCode == 316) // Left
	{
		sReturn = _("LeftPress");
	}
	else if (iCode == 317) // Up
	{
		sReturn = _("SelectPrevItem");
	}
	else if (iCode == 318) // Right
	{
		sReturn = _("RightPress");
	}
	else if (iCode == 319) // Down
	{
		sReturn = _("SelectNextItem");
	}
	else if (iCode == 13) // Enter
	{
		sReturn = _("Activate");
	}
	else if (iCode == 119)
	{
		sReturn = _("OpenInNew");
	}
	else if (iCode == 105)
	{
		sReturn = _("TreeFit");
	}
	else if (iCode == 'u')
	{
		sReturn = _("GoHome");
	}
	else if (iCode == 'a')
	{
		sReturn = _("AutoFitTree");
	}
	else if (iCode == 't')
	{
		sReturn = _("ToggleToolbar");
	}

	return sReturn;
}

void MainWindow::OnMove(wxMoveEvent &oEvent)
{
	treeFit();
}

