#include <wx/txtstrm.h>
#include "processbox.h"

enum
{
	O_PROCESS = wxID_HIGHEST,
	O_TIMER,
};

BEGIN_EVENT_TABLE(UProcessBox, wxTextCtrl)
	EVT_TIMER(O_TIMER, UProcessBox::OnTimer)
	EVT_END_PROCESS(O_PROCESS, UProcessBox::OnProcessDeath)
END_EVENT_TABLE()

UProcessBox::UProcessBox(wxWindow *oParent)
: wxTextCtrl(oParent, -1, _(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY)
{
	oTimer = new wxTimer(this, O_TIMER);
}

void UProcessBox::setCommand(const wxString &sCommand)
{
	sRunString = sCommand;
}

void UProcessBox::OnTimer(wxTimerEvent& WXUNUSED(oEvent))
{
	processInput();
}

void UProcessBox::run()
{
	Clear();
	wxProcess *oTempProcess = new wxProcess(this, O_PROCESS);
	oTempProcess->Redirect();
	long iPID = wxExecute(sRunString, wxEXEC_ASYNC, oTempProcess);
	if (iPID)
	{
		oProcess = oTempProcess;
		oTimer->Start(100);
	}
	else
	{
		delete oTempProcess;
	}
}

void UProcessBox::processInput()
{
	bool bHasInput = false;
	wxString sLine;
	
	if (oProcess->IsInputAvailable())
	{
		wxTextInputStream oInput(*oProcess->GetInputStream());
		sLine = oInput.ReadLine();
		if (sLine.Length() > 0)
		{
			AppendText(sLine + _("\n"));
			bHasInput = true;
		}
	}
	
	if (oProcess->IsErrorAvailable())
	{
		wxTextAttr oOldAttribute;
		wxColour oColour(255, 0, 0);
		wxTextAttr oAttribute(oColour);
		SetDefaultStyle(oAttribute);
		
		wxTextInputStream oInput(*oProcess->GetErrorStream());
		sLine = oInput.ReadLine();
		if (sLine.Length() > 0)
		{
			AppendText(sLine + _("\n"));
			bHasInput = true;
		}
		
		SetDefaultStyle(oOldAttribute);
	}
	
	if (bHasInput)
	{
		processInput();
	}
}

void UProcessBox::OnProcessDeath(wxProcessEvent &oEvent)
{
	processInput();
	oTimer->Stop();
	wxTextAttr oOldAttribute;
	wxColour oColour(128, 128, 128);
	wxTextAttr oAttribute(oColour);
	SetDefaultStyle(oAttribute);
	AppendText(_("*** Process ended. ***\n"));
	SetDefaultStyle(oOldAttribute);
}

