#include "settingswindow.h"

enum
{
	BTN_OK = wxID_HIGHEST,
	BTN_CANCEL,
};

BEGIN_EVENT_TABLE(SettingsWindow, wxFrame)
	EVT_BUTTON(BTN_OK, SettingsWindow::on_btnOK)
	EVT_BUTTON(BTN_CANCEL, SettingsWindow::on_btnCancel)
	EVT_CLOSE(SettingsWindow::on_this_close)
END_EVENT_TABLE()

SettingsWindow::SettingsWindow(wxFrame *oParent, const wxString &sTitle)
: wxFrame(oParent, -1, sTitle, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE)
{
	oPanel = new wxPanel(this, -1);
	oConfig = (wxConfig *)wxConfig::Get();
	
	// Sizer creation.
	oWindowSizer = new wxBoxSizer(wxVERTICAL);
	oMainSizer = new wxBoxSizer(wxVERTICAL);
	oFieldSizer = new wxFlexGridSizer(2, 2);
	oCloseSizer = new wxBoxSizer(wxHORIZONTAL);
	
	// Control creation.
	// oFileSettingsPanel = new FileSettingsPanel(oPanel); // To kill.
	
	lblDefaultApp = new wxStaticText(oPanel, -1, _("Default app: "));
	txtDefaultApp = new wxTextCtrl(oPanel, -1, _(""));
	
	lblScreenMargin = new wxStaticText(oPanel, -1, _("Screen margin: "));
	txtScreenMargin = new wxTextCtrl(oPanel, -1, _(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
	
	btnOK = new wxButton(oPanel, BTN_OK, _("OK"));
	btnCancel = new wxButton(oPanel, BTN_CANCEL, _("Cancel"));
	
	oGeneralBox = new wxStaticBox(oPanel, -1, _("General settings"));
	
	// Sizers that need to come into existence after something else is created.
	oGeneralSizer = new wxStaticBoxSizer(oGeneralBox, wxVERTICAL);
	
	// Sizer manipulation.
	oFieldSizer->Add(lblScreenMargin);
	oFieldSizer->Add(txtScreenMargin, 0, wxRIGHT, 4);
	oFieldSizer->Add(lblDefaultApp);
	oFieldSizer->Add(txtDefaultApp, 0, wxRIGHT, 4);
	
	oCloseSizer->Add(btnOK, 0, wxRIGHT, 4);
	oCloseSizer->Add(btnCancel);
	
	oGeneralSizer->Add(oFieldSizer, 0, wxALL, 4);
	
	// oMainSizer->Add(oFileSettingsPanel, 0, wxGROW); // To kill.
	oMainSizer->Add(oGeneralSizer, 0, wxALL, 4);
	oMainSizer->Add(oCloseSizer, 0, wxALIGN_RIGHT | wxALL, 4);
	
	oWindowSizer->Add(oMainSizer, 0, wxGROW);
	
	// Formalities.
	oPanel->SetSizer(oWindowSizer);
	oPanel->SetAutoLayout(true);
	oWindowSizer->Fit(this);
	
	// And finally...
	readSettings();
}

void SettingsWindow::on_btnOK(wxCommandEvent &oEvent)
{
	oConfig->Write(_("/DefaultApp"), txtDefaultApp->GetValue());
	oConfig->Write(_("/ScreenMargin"), txtScreenMargin->GetValue());
	oConfig->Flush();
	
	Close();
}

void SettingsWindow::on_btnCancel(wxCommandEvent &oEvent)
{
	Close();
}

void SettingsWindow::on_this_close(wxCloseEvent &oEvent)
{
	readSettings();
	Show(false);
}

void SettingsWindow::readSettings()
{
	txtDefaultApp->SetValue(oConfig->Read(_("/DefaultApp")));
	txtScreenMargin->SetValue(oConfig->Read(_("/ScreenMargin")));
}

