#include <wx/wx.h>
#include <wx/filename.h>
#include "constants.h"
#include "uthread.h"
#include "filetree.h"

UThread::UThread(UFiletree *oNewTree) : wxThread()
{
	this->oTree = oNewTree;
}

void UThread::checkChanges()
{
	wxMutexGuiEnter();
	// Make sure something is selected.
	if (!oTree->somethingSelected())
	{
		oTree->selectTreeItem(oTree->GetRootItem());
	}
	
	wxTreeItemId iRoot = oTree->GetRootItem();
	wxString sDir = oTree->getFullpath(iRoot);
	wxMutexGuiLeave();
	
	std::vector<wxTreeItemId> iReload;
	if (checkTime(sDir))
	{
		iReload.insert(iReload.begin(), iRoot);
	}
	wxTreeItemId iCurrent = iRoot;
	while (true)
	{
		wxMutexGuiEnter();
		if (!oTree->isValid(iCurrent))
		{
			wxMutexGuiLeave();
			break;
		}
		iCurrent = oTree->GetNextVisible(iCurrent);
		if (!iCurrent)
		{
			wxMutexGuiLeave();
			break;
		}
		sDir = oTree->getFullpath(iCurrent);
		wxMutexGuiLeave();
		if (checkTime(sDir))
		{
			iReload.insert(iReload.begin(), iCurrent);
		}
	}
	if (iReload.size() > 0)
	{
		for (unsigned int i = 0; i < iReload.size(); i++)
		{
			wxMutexGuiEnter();
			oTree->ReloadDir(iReload[i], true);
			oTree->oParentFrame->treeFit(true);
			wxMutexGuiLeave();
		}
	}
}

void *UThread::Entry()
{
	while (true)
	{
		wxThread::Sleep(100);
		this->checkChanges();
	}
	
	return NULL;
}

bool UThread::checkTime(const wxString &sDir)
{
	if (!wxDirExists(sDir))
	{
		return false;
	}
	
	bool bChanged = false;
	bool bFound = false;
	wxFileName oDir(sDir);
	wxDateTime oTime = oDir.GetModificationTime();
	for (unsigned int i = 0; i < sDirVector.size(); i++)
	{
		if (sDir == sDirVector[i])
		{
			bFound = true;
			if (oTime != oTimeVector[i])
			{
				oTimeVector[i] = oTime;
				bChanged = true;
			}
		}
	}
	if (!bFound)
	{
		sDirVector.push_back(sDir);
		oTimeVector.push_back(oTime);
	}
	return bChanged;
}

